//------------------------------------------------------------------------------
//SFWK Framework
//Copyright (C) 2011 SQLI

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Linq.Expressions;
using System.Runtime.Serialization;
using System.Xml.Serialization;

namespace SFWK.Core.ExpressionBuilder
{
    [DataContract]
    public class EditableLambdaExpression : EditableExpression
    {
        // Properties
        [DataMember]
        public EditableExpression Body
        {
            get;
            set;
        }

        [DataMember]
        public EditableExpressionCollection Parameters
        {
            get;
            set;
        }
        
        public override ExpressionType NodeType
        {
            get { return ExpressionType.Lambda; }
            set { }
        }

        // Ctors
        public EditableLambdaExpression()
        {
            Parameters = new EditableExpressionCollection();
        }      

        public EditableLambdaExpression(LambdaExpression lambEx)
            : base(lambEx.Type) 
        {
            Parameters = new EditableExpressionCollection();
            Body = EditableExpression.CreateEditableExpression(lambEx.Body);
            foreach (ParameterExpression param in lambEx.Parameters)
                Parameters.Add(EditableExpression.CreateEditableExpression(param));
        }

        // Methods
        public override Expression ToExpression()
        {
            Expression body = Body.ToExpression();
            List<ParameterExpression> parameters = new List<ParameterExpression>(Parameters.GetParameterExpressions());

            var bodyParameters = from edX in body.Nodes()
                                 where edX is ParameterExpression
                                 select edX;
            for (int i = 0; i < parameters.Count; i++)
            {
                var matchingParm = from parm in bodyParameters
                                   where (parm as ParameterExpression).Name == parameters[i].Name
                                      && (parm as ParameterExpression).Type == parameters[i].Type
                                   select parm as ParameterExpression;
                if (matchingParm.Count<ParameterExpression>() == 1)
                    parameters[i] = matchingParm.First<ParameterExpression>() as ParameterExpression;
            }

            return Expression.Lambda(Type, body, parameters);
        }
    }
}

