//------------------------------------------------------------------------------
//SFWK Framework
//Copyright (C) 2011 SQLI

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Linq.Expressions;
using System.Reflection;
using System.Runtime.Serialization;
using System.Xml.Serialization;

namespace SFWK.Core.ExpressionBuilder
{
    [DataContract]
    public class EditableMethodCallExpression : EditableExpression
    {                        
        // Properties
        [DataMember]
        public EditableExpressionCollection Arguments
        {
            get;
            set;
        }

        [XmlIgnore]
        public MethodInfo Method
        {
            get;
            set;
        }

        [DataMember]
        public string MethodName
        {
            get
            {
                return Method.ToSerializableForm();
            }
            set
            {
                Method = Method.FromSerializableForm(value);
            }
        }

        [DataMember]
        public EditableExpression Object
        {
            get;
            set;
        }
        
        public override ExpressionType NodeType
        {
            get;
            set;
        }

        // Ctors
        public EditableMethodCallExpression()
        {
        }

        public EditableMethodCallExpression(EditableExpressionCollection arguments, MethodInfo method, EditableExpression theObject, ExpressionType nodeType)
        {
            Arguments = arguments;
            Method = method;
            Object = theObject;
            NodeType = nodeType;
        }

        public EditableMethodCallExpression(IEnumerable<EditableExpression> arguments, MethodInfo method, Expression theObject, ExpressionType nodeType) :
            this(new EditableExpressionCollection(arguments), method, EditableExpression.CreateEditableExpression(theObject), nodeType)
        { 
        }
        
        public EditableMethodCallExpression(MethodCallExpression callEx) :
            this(new EditableExpressionCollection(callEx.Arguments),callEx.Method,EditableExpression.CreateEditableExpression(callEx.Object),callEx.NodeType)
        {
        }

        // Methods
        public override Expression ToExpression()
        {
            Expression instanceExpression = null;
            if (Object != null)
                instanceExpression = Object.ToExpression();

            return Expression.Call(instanceExpression, Method, Arguments.GetExpressions().ToArray<Expression>());
        }
    }
}

