//------------------------------------------------------------------------------
//SFWK Framework
//Copyright (C) 2011 SQLI

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Linq.Expressions;
using System.Reflection;
using System.Runtime.Serialization;
using System.Xml.Serialization;

namespace SFWK.Core.ExpressionBuilder
{
    [DataContract]
    public class EditableNewExpression : EditableExpression
    {
        // Properties                
        [XmlIgnore]
        public ConstructorInfo Constructor
        {
            get;
            set;
        }

        [DataMember]
        public EditableExpressionCollection Arguments
        {
            get;
            set;
        }

        [DataMember]
        public EditableMemberInfoCollection Members
        {
            get;
            set;
        }

        [DataMember]
        public string ConstructorName
        {
            get { return Constructor.ToSerializableForm(); }
            set { Constructor = Constructor.FromSerializableForm(value); }
        }
        
        public override ExpressionType NodeType
        {
            get { return ExpressionType.New; }
            set { }
        }

        // Ctors
        public EditableNewExpression()
        {
        }

        public EditableNewExpression(NewExpression newEx)
            : this(newEx.Constructor, new EditableExpressionCollection(newEx.Arguments), newEx.Members, newEx.Type)
        { }

        public EditableNewExpression(ConstructorInfo constructor, IEnumerable<EditableExpression> arguments, IEnumerable<MemberInfo> members, Type type)
            : this(constructor, new EditableExpressionCollection(arguments), members, type)
        { }

        public EditableNewExpression(ConstructorInfo constructor, EditableExpressionCollection arguments, IEnumerable<MemberInfo> members, Type type)
            : base(type)
        {
            Arguments = arguments;
            Constructor = constructor;            
            Members = new EditableMemberInfoCollection(members);
        }

        // Methods
        public override Expression ToExpression()
        {
            if (Constructor != null)
                return Expression.New(Constructor, Arguments.GetExpressions());
            else
                return Expression.New(Type);
        }
    }
}

