//------------------------------------------------------------------------------
//SFWK Framework
//Copyright (C) 2011 SQLI

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

using System;
using System.Linq;
using System.Collections.Generic;
using System.Text;
using System.Linq.Expressions;
using System.Runtime.Serialization;
using System.Xml.Serialization;

namespace SFWK.Core.ExpressionBuilder
{
    [DataContract]
    public class EditableParameterExpression : EditableExpression
    {
        // Members
        private static Dictionary<string, ParameterExpression> _usableParameters = new Dictionary<string, ParameterExpression>();

        // Properties
        [DataMember]
        public string Name 
        { 
            get; 
            set; 
        }

        public override ExpressionType NodeType
        {
            get { return ExpressionType.Parameter; }
            set { }
        }

        // Ctors
        public EditableParameterExpression()
        {
        }

        public EditableParameterExpression(ParameterExpression parmEx)
            : this(parmEx.Type, parmEx.Name)
        { }

        public EditableParameterExpression(Type type, string name)
            : base(type)
        {
            Name = name;
        }

        // Methods
        static public ParameterExpression CreateParameter(Type type, string name)
        {
            ParameterExpression parameter = null;
            string key = type.AssemblyQualifiedName + Environment.NewLine + name;
            if (_usableParameters.ContainsKey(key))
            {
                parameter = _usableParameters[key] as ParameterExpression;
            }
            else
            {
                parameter = Expression.Parameter(type, name);
                _usableParameters.Add(key, parameter);
            }
            return parameter;
        }

        public override Expression ToExpression()
        {
            return CreateParameter(Type, Name);
        }
    }
}

