//------------------------------------------------------------------------------
//SFWK Framework
//Copyright (C) 2011 SQLI

//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.

//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.

//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <http://www.gnu.org/licenses/>.
//------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.InteropServices;

namespace SFWK.Core.Test
{
    public static class TestUtils
    {
        public const string ALPHA = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        public const string NUMERIC = "0123456789";
        public const string ALPHA_NUMERIC = ALPHA + NUMERIC;
        public static Random random = new Random();

        //
        //  Creates a random password
        //    
        private static string _getRandomNumber(int length, string characterSet)
        {
            string randomData = "";
            int position = 0;
            byte[] data = new byte[length];
            int characterSetLength = characterSet.Length;
            System.Security.Cryptography.RandomNumberGenerator random = System.Security.Cryptography.RandomNumberGenerator.Create();
            random.GetBytes(data);
            for (int index = 0; (index < length); index++)
            {
                position = data[index];
                position = (position % characterSetLength);
                randomData = (randomData + characterSet.Substring(position, 1));
            }
            return randomData;
        }

        public static string GetRandomNumeric(int length)
        {
            return _getRandomNumber(length, NUMERIC);
        }
        public static string GetRandomAlphaString(int length)
        {
            return _getRandomNumber(length, ALPHA);
        }
        public static string GetRandomAlphaNumericString(int length)
        {
            return _getRandomNumber(length, ALPHA_NUMERIC);
        }

        public static T SelectRandom<T>(List<T> lst)
        {
            return lst[TestUtils.random.Next(0, lst.Count - 1)];
        }
       
        /// <summary>
        /// RandomDate
        /// </summary>
        /// <returns></returns>
        
        public static DateTime CreateRandomDate()
        {
            
            int day = random.Next(1, 28);
            int month = random.Next(1, 12);
            int year = random.Next(1980, DateTime.Now.Year);
            DateTime date = new DateTime(year, month, day);
            return date;
        }


        public static int CreateRandomInt()
        {
            return Convert.ToInt32(GetRandomNumeric(Marshal.SizeOf(typeof(int))));
        }

        public static long CreateRandomLong()
        {
            return Convert.ToInt64(GetRandomNumeric(Marshal.SizeOf(typeof(long))));
        }

        public static double CreateRandomDouble()
        {
            return random.NextDouble();
        }

        public static decimal CreateRandomDecimal()
        {
            return Convert.ToDecimal(GetRandomNumeric(Marshal.SizeOf(typeof(decimal))));
        }


        public static bool CreateRandomBool()
        {
            return Convert.ToBoolean(random.Next(0,1));
        }

    }
}

